import DateRangePicker from 'tiny-date-picker/dist/date-range-picker';
import moment from 'moment';
export class CwcDatepicker {
    constructor() {
        this.lang = 'en';
        this.format = 'MM.DD.YYYY';
        this.langConstants = {
            en: {
                days: 'Sun_Mon_Tue_Wed_Thu_Fri_Sat'.split('_'),
                months: 'January_February_March_April_May_June_July_August_September_October_November_December'.split('_'),
                today: 'Today',
                clear: 'Clear',
                close: 'Close',
                startDate: 'Start date',
                endDate: 'End date'
            },
            de: {
                days: 'So._Mo._Di._Mi._Do._Fr._Sa.'.split('_'),
                months: 'Januar_Februar_März_April_Mai_Juni_Juli_August_September_Oktober_November_Dezember'.split('_'),
                today: 'Heute',
                clear: 'Klar',
                close: 'Schließen',
                startDate: 'Frühestens',
                endDate: 'Spätestens'
            }
        };
    }
    componentDidLoad() {
        this.container = this.el.querySelector('.ex-inputs-picker');
        this.rangeInputElement = this.el.querySelector('.ex-input-daterange');
        const self = this;
        this.dp = DateRangePicker.DateRangePicker(this.container, {
            startOpts: {
                start: this.startDate ? new Date(this.startDate) : '',
                end: this.endDate ? new Date(this.endDate) : '',
                lang: this.langConstants[this.lang] || this.langConstants['en']
            }
        }).on('statechange', function (_, rp) {
            // Update the inputs when the state changes
            self.range = rp.state;
            self.rangeInputElement.value = self.formInputValueString(self.range.start, self.range.end);
            self.statechange.emit({
                start: self.range.start,
                end: self.range.end
            });
            self.onChange.emit({
                id: self.id,
                value: {
                    endDate: self.range.end ? self.endDateValueString(self.range.end) : null,
                    startDate: self.range.start ? self.startDateValueString(self.range.start) : null
                }
            });
        });
        // When the inputs gain focus, show the date range picker
        this.rangeInputElement.addEventListener('focus', () => this.showPicker());
        this.rangeInputElement.addEventListener('change', () => {
            const parsed = this.parseInputString(this.rangeInputElement.value);
            const dateStart = new Date(parsed[0]), dateEnd = new Date(parsed[1]);
            this.dp.setState({
                start: !isNaN(dateStart.getTime()) ? dateStart : '',
                end: !isNaN(dateEnd.getTime()) ? dateEnd : ''
            });
        });
        // If focus leaves the root element, it is not in the date
        // picker or the inputs, so we should hide the date picker
        // we do this in a setTimeout because redraws cause temporary
        // loss of focus.
        let previousTimeout;
        this.el.addEventListener('focusout', function hidePicker() {
            clearTimeout(previousTimeout);
            previousTimeout = setTimeout(function () {
                if (!self.el.contains(document.activeElement)) {
                    self.container.classList.remove('ex-inputs-picker-visible');
                }
            }, 10);
        });
    }
    parseInputString(value) {
        const values = value.split('-');
        return values.map(val => val.trim());
    }
    formInputValueString(startDate, endDate) {
        return `${this.startDateValueString(startDate)} - ${this.endDateValueString(endDate)}`;
    }
    startDateValueString(startDate) {
        return startDate ? moment(startDate).format(this.format) : this.langConstants[this.lang].startDate;
    }
    endDateValueString(endDate) {
        return endDate ? moment(endDate).format(this.format) : this.langConstants[this.lang].endDate;
    }
    showPicker() {
        this.container.classList.add('ex-inputs-picker-visible');
    }
    render() {
        return (h("div", null,
            h("div", { class: "form-group ex-inputs" },
                h("label", { class: "control-label" }, this.label),
                h("input", { id: this.id, class: "form-control ex-input-daterange", placeholder: `${this.langConstants[this.lang].startDate} - ${this.langConstants[this.lang].endDate}` }),
                h("div", { class: "ex-inputs-picker" }))));
    }
    static get is() { return "cwc-datepicker"; }
    static get properties() { return {
        "el": {
            "elementRef": true
        },
        "element": {
            "elementRef": true
        },
        "endDate": {
            "type": String,
            "attr": "end-date"
        },
        "format": {
            "type": String,
            "attr": "format"
        },
        "id": {
            "type": String,
            "attr": "id"
        },
        "label": {
            "type": String,
            "attr": "label"
        },
        "lang": {
            "type": String,
            "attr": "lang"
        },
        "startDate": {
            "type": String,
            "attr": "start-date"
        }
    }; }
    static get events() { return [{
            "name": "statechange",
            "method": "statechange",
            "bubbles": true,
            "cancelable": true,
            "composed": true
        }, {
            "name": "onChange",
            "method": "onChange",
            "bubbles": true,
            "cancelable": true,
            "composed": true
        }]; }
    static get style() { return "/**style-placeholder:cwc-datepicker:**/"; }
}
